#ifndef STANDARD_CLASS_PROP_EDITOR_H
#include "StandardClassPropEditor.h"
#endif

#define I_X_LABEL		'xLbl'
#define I_X_PREF_RESIZE	'sPSz'
#define I_X_ENABLE		'xEbl'

#define MARGE	6
#define TEXT_HAUT	20

class BControlPropEditor : public StandardClassPropEditor {
public:
	BControlPropEditor(BArchivable *settarget);
	~BControlPropEditor();
	virtual void	AttachedToWindow();
	virtual void	MessageReceived(BMessage* msg);
	
	virtual void		SetUpData(BArchivable* changed);
private:
	BTextControl*	labelCntl;
	BCheckBox		*enableCntl;
};

BControlPropEditor::BControlPropEditor(BArchivable *settarget):
	StandardClassPropEditor(BRect(0,0,300,40), "BControl", settarget) {
	BRect	r;
	
	r = Bounds();
	r.top = 18;
	r.left = MARGE;
	r.bottom = r.top + TEXT_HAUT;
	r.right = r.left + 100;
	labelCntl = new BTextControl(r, "labelCntl", "Label", "", new BMessage(I_X_LABEL));
	labelCntl->SetDivider(labelCntl->StringWidth(labelCntl->Label()) + 3);
	AddChild(labelCntl);
		
	enableCntl = new BCheckBox(r, "enableCntl", "Enable", new BMessage(I_X_ENABLE));
	enableCntl->ResizeToPreferred();
	enableCntl->MoveBy(Bounds().right - MARGE - enableCntl->Frame().right, 0);
	AddChild(enableCntl);
}

BControlPropEditor::~BControlPropEditor() {
	BWindow*	w = ((BView*) target)->Window();
	BWindow*	self = Window();
	if (w->LockingThread() == 0 || (self && w->LockingThread() == self->Thread()))
		MessageReceived(&BMessage(I_X_LABEL));
}

void	BControlPropEditor::AttachedToWindow() {
	inherited::AttachedToWindow();
	labelCntl->SetTarget(this);
	enableCntl->SetTarget(this);
	
	BButton*	but;
	BRect	r = labelCntl->Frame();
	r.left = r.right + MARGE;
	r.right = enableCntl->Frame().left - MARGE;
	r.OffsetBy(0,-4);
	but = new BButton(r, "resizeTo", "Preferred size", new BMessage(I_X_PREF_RESIZE));
	but->ResizeToPreferred();
	AddChild(but);
	but->SetTarget(this);
}
void	BControlPropEditor::SetUpData(BArchivable* changed) {
	BControl* c = cast_as(target, BControl);
	char	sval[128];
	if (!changed || changed == target && c && c->Window()) {
		c->Window()->Lock();
		labelCntl->SetText(c->Label());
		enableCntl->SetValue(c->IsEnabled() ? B_CONTROL_ON : B_CONTROL_OFF);
		c->Window()->Unlock();
	}
}

void	BControlPropEditor::MessageReceived(BMessage* msg) {
	BControl*	c;
	c = cast_as(target, BControl);
	switch(msg->what) {
		case I_X_LABEL:
			if (c && c->Window()) {
				if (c->Window()->LockWithTimeout(1000000) != B_OK) break;
				c->SetLabel(labelCntl->Text());
				c->Invalidate();
				c->Window()->Unlock();
				CommitChange();
			}
			break;
		case I_X_PREF_RESIZE:
			MessageReceived(&BMessage(I_X_LABEL));
			if (c && c->Window()) {
				if (c->Window()->LockWithTimeout(1000000) != B_OK) break;
				c->ResizeToPreferred();
				c->Invalidate();
				c->Window()->Unlock();
				CommitChange();
			}
			break;
		case I_X_ENABLE:
			if (c && c->Window()) {
				if (c->Window()->LockWithTimeout(1000000) != B_OK) break;
				c->SetEnabled((enableCntl->Value() ==  B_CONTROL_ON) ? true : false);
				c->Invalidate();
				c->Window()->Unlock();
				CommitChange();
			}
			break;
		default:
			inherited::MessageReceived(msg);
	}
}

#pragma export on
BView* CreateEditor(BArchivable* trytarget);
#pragma export reset

BView* CreateEditor(BArchivable* trytarget) {
	BControl*	c = cast_as(trytarget, BControl);
	if (c) {
		BControlPropEditor*	cont;
		cont = new BControlPropEditor(trytarget);
		return cont;
	}
	else 
		return NULL;
}